# Tests for student related reports
from zope.interface.verify import verifyClass, verifyObject
from waeup.kofa.students.reports import (
    get_students_by, StudentsReport, IStudentsReport)
from waeup.kofa.students.tests.test_catalog import CatalogTestSetup
from waeup.kofa.testing import FunctionalLayer

class StudentsReportTests(CatalogTestSetup):

    layer = FunctionalLayer

    states = ('created', 'admitted', 'clearance started',
              'clearance requested', 'cleared', 'school fee paid',
              'returning', 'courses registered', 'courses validated', 'Total')

    def test_iface(self):
        # ensure we fullfill interface contracts
        obj = StudentsReport(2010, 'Undergraduate Full-Time')
        verifyClass(IStudentsReport, StudentsReport)
        verifyObject(IStudentsReport, obj)
        return

    def test_repr(self):
        # exec(students_report.__repr__()) <=> students_report
        obj1 = StudentsReport(2010, 'Undergraduate Full-Time', author='Bob')
        self.assertEqual(
            obj1.__repr__(),
            "StudentsReport(2010, 'Undergraduate Full-Time', author='Bob')")
        obj_from_str = eval(obj1.__repr__())
        self.assertEqual(obj_from_str.session, '2010/2011')
        self.assertEqual(obj_from_str.mode, 'Undergraduate Full-Time')
        self.assertEqual(obj_from_str.author, 'Bob')
        return

    #def test_str(self):
    #    # StudentsReport provide a nice string representation
    #    obj = StudentsReport(2010, 'Undergraduate Full-Time', author='Bob')
    #    self.assertEqual(
    #        obj.__str__()[:65],
    #        'Students Report '
    #        '[session=2010/2011, mode=Undergraduate Full-Time]')
    #    return

    def test_title(self):
        obj = StudentsReport(2010, 'Undergraduate Full-Time', author='Bob')
        self.assertTrue(
            obj.title.startswith(
            'StudentsReport_2010_2011_Undergraduate_Full-Time_'))
        return

    def test_get_students_by_session_simple(self):
        # we can get a table with one student
        result1 = get_students_by(2010, 'Undergraduate Full-Time')
        result2 = get_students_by(2009, 'Undergraduate Full-Time')
        self.assertEqual(
            result1,
            ((u'fac1', u'Total',),
             self.states,
             ((1, 0, 0, 0, 0, 0, 0, 0, 0, 1),
              (1, 0, 0, 0, 0, 0, 0, 0, 0, 1),)))
        self.assertEqual(
            result2,
            ((u'fac1', u'Total'),
             self.states,
             ((0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0),)))
        return

    def test_get_students_by_session_multiple(self):
        # we can get a table with several students
        self.create_cert('fac2', 'dept2', 'CERT2')
        result1 = get_students_by(2010, 'Undergraduate Full-Time')
        result2 = get_students_by(2009, 'Undergraduate Full-Time')
        self.assertEqual(
            result1,
            ((u'fac1', u'fac2', u'Total'),
             self.states,
             ((1, 0, 0, 0, 0, 0, 0, 0, 0, 1),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (1, 0, 0, 0, 0, 0, 0, 0, 0, 1),)))
        self.assertEqual(
            result2,
            ((u'fac1', u'fac2', u'Total'),
             self.states,
             ((0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
              (0, 0, 0, 0, 0, 0, 0, 0, 0, 0),)))
        return

    def test_create_pdf(self):
        self.create_cert('FAC2', 'dept2', 'CERT2')
        report = StudentsReport(2010, 'Undergraduate Full-Time')
        result = report.create_pdf()
        self.assertTrue(result.startswith('%PDF-'))
        return
